( ****h* 4e4th/onewire.f
( NAME
(   onewire.f
( SYNOPSIS
(   Example high-level Forth functions for Dallas 1-wire devices
( DESCRIPTION
(
( USES
(   Uses the following kernel functions (provided by 4e-onewire.s43)
(   OWRESET [ -- f ]  Initialize 1-wire devices; return true if present
(   OWSLOT [ c -- c' ] Write and read one bit to/from 1-wire.
(
( NOTES
(   See the file onewire.txt for documentation of these words. 
( COPYRIGHT
(   [c] 2012 Bradford J. Rodriguez.
( 
(   This program is free software; you can redistribute it and/or modify
(   it under the terms of the GNU General Public License as published by
(   the Free Software Foundation; either version 3 of the License, or
(   [at your option] any later version.
(
(   This program is distributed in the hope that it will be useful,
(   but WITHOUT ANY WARRANTY; without even the implied warranty of
(   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
(   GNU General Public License for more details.
(
(   You should have received a copy of the GNU General Public License
(   along with this program.  If not, see <http://www.gnu.org/licenses/>.
(
(   Commercial inquiries should be directed to the author at 
(   115 First St., #105, Collingwood, Ontario L9Y 4W3 Canada
(   or via email to bj@camelforth.com
( ******


( Basic 1-wire operations )

: OWTOUCH ( c1 -- c2 ) 
    OWSLOT OWSLOT OWSLOT OWSLOT OWSLOT OWSLOT OWSLOT OWSLOT ;
HEX 
: OWPUT ( c -- ) OWTOUCH DROP ;  
: OWGET ( -- c ) 0FF OWTOUCH ;

( SHOWID should be used ONLY if there is a single 1-wire device attached.)
: SHOWID  OWRESET IF
    033 OWPUT OWGET . OWGET . OWGET . OWGET .
    OWGET . OWGET . OWGET . OWGET . 
  THEN ;

( Maxim 1-wire ROM Search algorithm )
( per AN937 "Book of iButton Standards", figure 5-3 )

HEX
VARIABLE LASTDISC               ( used as byte variable )
LASTDISC 1+ CONSTANT DONEFLAG   ( used as byte variable )

VARIABLE ROMBIT                 ( used as byte variable, 1..64 )
ROMBIT 1+ CONSTANT DISCMARK     ( used as byte variable )

VARIABLE ROMID  6 ALLOT         ( 8 byte array )

: !ROMBIT ( f -- )
    ROMBIT C@ 1- 8 /MOD ( -- f bit# byte# )
    ROMID +             ( -- f bit# addr )
    1 ROT LSHIFT        ( -- f addr bitmask )
    ROT IF              ( f true, set bit )
        OVER C@ OR SWAP C!
    ELSE                ( f false, clear bit )
        INVERT OVER C@ AND SWAP C!
    THEN
;

: @ROMBIT ( -- f )
    ROMBIT C@ 1- 8 /MOD ( -- bit# byte# )
    ROMID + C@          ( -- bit# byte )
    1 ROT LSHIFT        ( -- byte bitmask )
    AND
;    

: NEWSEARCH   0 LASTDISC ! ;    ( clear LASTDISC and DONEFLAG )

: ROMSEARCH ( -- f )   ( Returns 0 or 1 )
    0                   ( default return value )
    DONEFLAG C@ IF
        0 DONEFLAG C!  
        EXIT
    THEN
    OWRESET IF          ( presence signal detected? )

        1 ROMBIT C!         ( yes: set ROM bit index to 1 )
        0 DISCMARK C!       ( set discrepancy marker to 0 )
        0F0 OWPUT           ( send search command on bus )
        BEGIN
            03 OWSLOT OWSLOT    ( read two bits: ba000000 )
            DUP 0C0 = IF        ( bitA = bitB = 1?)
                DROP
                0 LASTDISC C! 
                EXIT
            ELSE DUP 0= IF      ( bitA = bitB = 0?)
                DROP
                ROMBIT C@ LASTDISC C@ = IF
                    1 !ROMBIT
                ELSE ROMBIT C@ LASTDISC C@ > IF
                    0 !ROMBIT
                    ROMBIT C@ DISCMARK C!
                ELSE @ROMBIT 0= IF
                    ROMBIT C@ DISCMARK C!
                THEN THEN THEN
            ELSE
                040 AND         ( bit A value )
                !ROMBIT
            THEN THEN
            @ROMBIT IF 1 ELSE 0 THEN OWSLOT DROP ( send ROM bit to bus )
            ROMBIT C@ 1+ DUP ROMBIT C!
        040 > UNTIL
        DISCMARK C@ DUP LASTDISC C!
        0= IF 
            1 DONEFLAG C!
        ELSE
            DROP 1      ( set return value to true )
        THEN
        
    ELSE ( no presence signal )
        0 LASTDISC C!
    THEN
;

( Demonstrates how to use ROMSEARCH to find all attached devices )

: SHOWIDS ( -- )
   NEWSEARCH
   BEGIN
      ROMSEARCH
      CR ROMID 8 + ROMID DO I C@ 3 U.R LOOP
   0= UNTIL
   CR
;

( Demonstrates how to define a named word for a 1-wire ID )
( This is the ID for one of my DS18b20 sensors )
HEX
CREATE SENSOR1  28 C, C7 C, 7E C, D5 C, 2 C, 0 C, 0 C, 4 C,

( This version of SENDID expects the address of an 8-byte ID )
( Start an addressed command.  This sends RESET, Match ROM [55h], )
( and the 8 bytes of ROMID.  It should be followed by a DS18B20   )
( function command. )

: SENDID ( addr -- )
   OWRESET IF
      55 OWPUT    ( send Match ROM command )
      8 OVER + SWAP DO  I C@ OWPUT  LOOP   ( send 8 id bytes )
   ELSE ." failed" DROP THEN
;

( Function commands that can follow SENDID )

: READSCRATCH ( addr -- )  ( display 9 bytes of scratchpad )
   SENDID
   0BE OWPUT  OWGET . OWGET . OWGET . OWGET .
   OWGET . OWGET . OWGET . OWGET . OWGET . 
;

: CONVERT ( addr -- )  ( start a temperature conversion )
   SENDID		( remember to allow 750 msec for conv.)
   44 OWPUT
;

: READTEMP ( addr -- n )   ( read first 2 bytes of scratchpad )
   SENDID
   0BE OWPUT  OWGET ( LSbyte )  OWGET ( MSbyte )
   8 LSHIFT OR    ( convert 2 bytes to single 16-bit value )
;

( Demonstrates how to scale temperature and format for PAD )
( Expects the value 'n' returned by READTEMP.  N=8 corresponds )
( to 0.5 degrees Celsius.  Format is [-]nn.n )

: TEMP>PAD ( n -- )
   5 8 */    ( convert temp to tenths degree Celsius )
   DECIMAL
   <# DUP ABS 0 # 2E HOLD # # ROT SIGN #>    ( -- addr n )
   PAD SWAP CMOVE   ( copy text string to PAD )
;

( To create a software delay between CONVERT and READTEMP. )
( Use e.g.  500 MSEC  for a 500 msec delay. )
( The inner DO LOOP is about 1 msec. )
DECIMAL
: MSEC ( n -- ) 0 DO  728 0 DO LOOP  LOOP ;

( Example usage: )
SENSOR1 CONVERT  750 MSEC  SENSOR1 READTEMP TEMP>PAD
